@@ -8,7 +8,7 @@ package Linux::DesktopFiles;
 #use strict;
 #use warnings;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 sub new {
     my ($class, %opts) = @_;
@@ -100,25 +100,13 @@ sub new {
         warn "Invalid option or value: $key";
     }
 
-    return $self;
+    $self;
 }
 
-sub iterate_desktop_files {
-    my ($self, $code) = @_;
-
-    foreach my $dir (@{$self->{desktop_files_paths}}) {
-        opendir(my $dir_h, $dir) or next;
-        foreach my $file (readdir $dir_h) {
-            $self->$code("$dir/$file") if substr($file, -8) eq '.desktop';
-        }
-        closedir $dir_h;
-    }
-}
-
-sub get_desktop_files {
+sub _init_icon_database {
     my ($self) = @_;
-    $self->iterate_desktop_files(sub { push @{$self->{DESKTOP_FILES}}, $_[1] });
-    wantarray ? @{$self->{DESKTOP_FILES}} : $self->{DESKTOP_FILES};
+    require GDBM_File;
+    tie %{$self->{icons_db}}, 'GDBM_File', $self->{icon_db_filename}, &GDBM_File::GDBM_WRCREAT, 0640;
 }
 
 sub get_icon_theme_name {
@@ -248,25 +236,24 @@ sub get_icon_path {
     $icon;    # return the icon
 }
 
-sub parse_desktop_file {
-    my ($self, $file) = @_;
+sub get_desktop_files {
+    my ($self) = @_;
 
-    $self->_clean_categories();
-    $self->_store_info(-f $file ? $file : return);
-
-    foreach my $item (values %{$self->{categories}}) {
-        if (ref $item eq 'ARRAY') {
-            return $item->[0];
+    my @desktop_files;
+    foreach my $dir (@{$self->{desktop_files_paths}}) {
+        opendir(my $dir_h, $dir) or next;
+        foreach my $file (readdir $dir_h) {
+            push(@desktop_files, "$dir/$file") if substr($file, -8) eq '.desktop';
         }
+        closedir $dir_h;
     }
-
-    return;
+    wantarray ? @desktop_files : \@desktop_files;
 }
 
-sub _store_info {
-    my $self = shift;
+sub parse {
+    my ($self, $file_data, @desktop_files) = @_;
 
-    foreach my $desktop_file (@_) {
+    foreach my $desktop_file(@desktop_files) {
 
         if (defined $self->{skip_filename_re}) {
             substr($desktop_file, rindex($desktop_file, '/') + 1) =~ /$self->{skip_filename_re}/o && next;
@@ -340,52 +327,36 @@ sub _store_info {
 
         if (scalar(@categories)) {
             foreach my $category (@categories) {
-                push @{$self->{categories}{$category}}, {map { $_ => $info{$_} } @{$self->{keys_to_keep}}};
+                push @{$file_data->{$category}}, {map { $_ => $info{$_} } @{$self->{keys_to_keep}}};
             }
         }
         elsif ($self->{keep_unknown_categories}) {
-            push @{$self->{categories}{$self->{unknown_category_key}}},
+            push @{$file_data->{$self->{unknown_category_key}}},
               {map { $_ => $info{$_} } @{$self->{keys_to_keep}}};
         }
     }
 }
 
-sub _clean_categories {
-    my ($self) = @_;
-    @{$self->{categories}}{keys %{$self->{categories}}} = ();
-}
-
-sub _categories {
-    my ($self) = @_;
-
-    $self->{keep_empty_categories} || do {
-        foreach my $key (keys %{$self->{categories}}) {
-            defined($self->{categories}{$key}) || delete $self->{categories}{$key};
-        }
-    };
-
-    $self->{categories};
+sub parse_desktop_file {
+    my ($self, $file) = @_;
+    my %file_data;
+    $self->parse(\%file_data, $file);
+    %file_data ? (values %file_data)[0][0] : ();
 }
 
 sub parse_desktop_files {
     my ($self) = @_;
-
-    $self->_clean_categories();
-    $self->iterate_desktop_files(\&_store_info);
-    $self->_categories();
-}
-
-sub _init_icon_database {
-    my ($self) = @_;
-
-    require GDBM_File;
-    tie %{$self->{icons_db}}, 'GDBM_File', $self->{icon_db_filename}, &GDBM_File::GDBM_WRCREAT, 0640;
+    my %categories;
+    $self->parse(\%categories, $self->get_desktop_files);
+    \%categories;
 }
 
 1;
 
 __END__
 
+=encoding utf8
+
 =head1 NAME
 
 Linux::DesktopFiles - Get and parse the Linux desktop files.
@@ -399,7 +370,7 @@ Linux::DesktopFiles - Get and parse the Linux desktop files.
 
 =head1 DESCRIPTION
 
-The C<Linux::DesktopFiles>, a very fast an simple way to parse the Linux desktop files.
+The C<Linux::DesktopFiles>, a very fast and simple way to parse the Linux desktop files.
 
 =head1 CONSTRUCTOR METHODS
 
@@ -425,7 +396,7 @@ The following options correspond to attribute methods described below:
    case_insensitive_cats       0
    keep_empty_categories       0
    strict_icon_dirs            0
-   use_current_theme_icons     0/1
+   use_current_theme_icons     0
    terminalize                 0
    terminal                    $ENV{TERM}
    home_dir                    $ENV{HOME} || $ENV{LOGDIR}
@@ -466,7 +437,7 @@ hash reference when calling C<$obj-E<gt>parse_desktop_files>.
 
 =item categories => [qw(Graphics Network AudioVideo ...)]
 
-Any of the valid categories from the desktop files. Any category not listed,
+Any of the valid categories from the desktop files. Any category not listed
 will be ignored.
 
 =back
@@ -615,10 +586,6 @@ Example:
 
 =over 4
 
-=item $obj->iterate_desktop_files(\&code_ref)
-
-Iterate over desktop files, one file at a time.
-
 =item $obj->get_desktop_files()
 
 Get all desktop files. In list context returns a list, but in scalar context,
@@ -637,6 +604,10 @@ If I<abs_icon_paths> is set to a false value, it strips the extension name of
 the icon (if any), and returns the icon name. If the icon name is undefined, it
 returns an empty string.
 
+=item $obj->parse(\%hash, @desktop_files)
+
+Parse a list of desktop files into a HASH ref.
+
 =item $obj->parse_desktop_file("filename")
 
 It returns a HASH reference which contains the I<keys_to_keep> and the corresponding
@@ -655,15 +626,19 @@ The returned HASH reference might look something like this:
           network => [ {Exec => "...", Name => "..."}, {Exec => "...", Name => "..."} ],
         }
 
+This function is equivalent with:
+
+    $obj->parse(\%hash, $obj->get_desktop_files);
+
 =back
 
 =head1 AUTHOR
 
-Daniel "Trizen" Suteu, E<lt>trizenx@gmail.comE<gt>
+Daniel "Trizen" Șuteu, E<lt>trizenx@gmail.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2012-2013
+Copyright (C) 2012-2013, 2015
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.14.2 or,
